//---------------------------------------------------------------------------
//  Mental Software Update Checker Unit.
//  Copyright (c) 2004 Mental Studio - http://mental.mentsu.com
//  Author : Raptor - raptor@mentsu.com
//  Create : Dec.08-04
//---------------------------------------------------------------------------
unit MUpdChkr;

interface

uses
  SysUtils, Classes, Forms, SyncObjs, IdHTTP;

type
  TMUpdateChecker = class(TComponent)
  private
    FCrit : TCriticalSection;

    FURL: String;
    FSoftwareName: String;
    FAutoCheck  : Boolean;
    FOnCheckDone: TNotifyEvent;

  protected
    Procedure Loaded; Override;
    Procedure CheckDone( Sender : TObject );
  public
    constructor Create( aOwner : TComponent ); Override;
    destructor Destroy; Override;
    Procedure CheckNow;
  published
    Property URL : String Read FURL Write FURL;
    Property SoftwareName : String Read FSoftwareName Write FSoftwareName;
    Property AutoCheck : Boolean Read FAutoCheck Write FAutoCheck Default false;
    Property OnCheckDone : TNotifyEvent Read FOnCheckDone Write FOnCheckDone;
  end;

procedure Register;

implementation

Uses
    MtCommon;

type
    TMUpdateThread = class(TThread)
    private
        FURL : String;
        FSoftwareName : String;
        FVer : TMFileVersion;

    protected
        Procedure Execute; Override;

    public
        constructor Create( aURL, aSoftwareName : String );
        destructor Destroy; override;

        Property Version : TMFileVersion Read FVer;
    end;

procedure Register;
begin
  RegisterComponents('MentalCtrls', [TMUpdateChecker]);
end;

//  Get new version thread
{ TMUpdateThread }

constructor TMUpdateThread.Create( aURL, aSoftwareName : String );
begin
    FURL := aURL;
    FSoftwareName := aSoftwareName;
    FVer := TMFileVersion.Create( );
    FreeOnTerminate := true;
    inherited Create( true );
end;

destructor TMUpdateThread.Destroy;
begin
    FVer.Free;
    inherited;
end;

procedure TMUpdateThread.Execute;
Var
    webConn : TIdHTTP;
    ss : TStrings;
    s : String;
begin
    webConn := TIdHTTP.Create( Nil );
    ss := TStringList.Create( );
    Try
        Try
            ss.Text := webConn.Get( FURL );
        Except
            ss.Text := '';
        End;
        s := ss.Values[FSoftwareName];
        If ( s <> '' ) Then
            FVer.VerStr := s;
    Finally
        ss.Free;
        webConn.Free;
    End;
end;

{ TMUpdateChecker }

constructor TMUpdateChecker.Create( aOwner : TComponent );
Begin
    Inherited;
    FCrit := TCriticalSection.Create( );
    FAutoCheck := false;
End;

destructor TMUpdateChecker.Destroy;
Begin
    FCrit.Free;
    Inherited;
End;

procedure TMUpdateChecker.Loaded;
begin
    inherited;
    If ( FAutoCheck ) Then
        CheckNow( );
end;

procedure TMUpdateChecker.CheckDone( Sender : TObject );
Var
    fv : TMFileVersion;
begin
    fv := TMFileVersion.Create( );
    Try
        fv.GetVersionFromFile( Application.ExeName );
        If ( ( ( Sender As TMUpdateThread ).Version.Compare( fv ) > 0 )
            AND ( Assigned( FOnCheckDone ) ) ) Then
            FOnCheckDone( Self );
    Finally
        fv.Free;
        FCrit.Leave( );
    End;
end;

Procedure TMUpdateChecker.CheckNow;
Begin
    With TMUpdateThread.Create( FURL, FSoftwareName ) Do
    Begin
        OnTerminate := CheckDone;
        Resume( );
    End;
    FCrit.Enter( );
End;

end.
 