//---------------------------------------------------------------------------
//  MtCommon unit. ( Delphi )
//  Copyright (c) 2003 Mental Studio - http://mental.mentsu.com
//  Author : Raptor - raptor@mentsu.com
//  Create : Sep.8-02
//  Last modify : Dec.4-03, Jun.3-04, Dec.3-04

//  ServiceControl
//    ڲ Windows NT/2000 ķ
//  GetVisibleRect
//    ȡóС
//  TMCommandString :
//    ڴضʽCommand:Param1=Value1;Param2=Value2
//  TMFileVersion :
//    ڴаİ汾ϢԴTAboutDialogؼ
//  TMCustomApplication :
//    ڴһϢѭԱûĬϢѭĵطDLḶ߳ʹ
//  WindowsϢҪдϢ
//---------------------------------------------------------------------------

unit MtCommon;

interface

Uses
	Windows, SysUtils, Classes, Messages, WinSvc, Forms;

Type
    TMCommandString = class(TObject)
    private
        FCmdStr  : String;
        FCommand : String;
        FParams  : TStrings;

        procedure SetCmdStr(const Value: String);
        procedure SetCommand(const Value: String);
        Function  GetCount : Integer;
        function  GetParam(aName: String): String;
        procedure SetParam(aName: String; const Value: String);

        Procedure Split;
        Procedure Merge;
    	function GetName(aIndex: Integer): String;
    public
      constructor Create;
      destructor Destroy; override;

      procedure LoadFromStream( aStream : TStream );
      procedure SaveToStream( aStream : TStream );

      Property CmdStr  : String read FCmdStr write SetCmdStr;
      Property Command : String read FCommand write SetCommand;
      Property Count   : Integer Read GetCount;
      Property Names[aIndex : Integer] : String Read GetName;
      Property Params[aName : String]  : String Read GetParam Write SetParam;
    end;

	TMFileVersion = class
    private
    	FMajor   : Integer;
        FMinor   : Integer;
        FRelease : Integer;
        FBuild   : Integer;

        Function  GetVerStr : String;
        Procedure SetVerStr( aVerStr : String );
	public
		constructor Create;
		destructor Destroy; override;

        Procedure GetVersionFromFile( aFileName : String );
        Function  Compare( aVer : TMFileVersion ) : Integer;

        Property VerStr : String read GetVerStr write SetVerStr;
    End;

    TMCustomApplication = class( TObject )
    Protected
        FHandle : HWND;

        Procedure WndFunc( Var aMsg : TMessage ); Virtual;
    public
        constructor Create;
        destructor Destroy; override;

        Property Handle : HWND Read FHandle;
    end;

Function ServiceControl(aServiceName: String; aOp: Integer) : Boolean;
Procedure GetVisibleRect( Var aRect : TRect );

implementation

//   Windows NT/2000 ķ
//  aOp = -1 ( Query ) return false ( Stopped ) true ( Started )
//  aOp = 0  ( Stop  ) return false ( Fail    ) true ( Success )
//  aOp = 1  ( Start ) ͬ
Function ServiceControl(aServiceName: String; aOp: Integer) : Boolean;
Var
	scm : Cardinal;
    svc : Cardinal;
    sts : _SERVICE_STATUS;
    arg : PChar;
begin
	Result := false;
	If ( ( GetVersion AND $80000000 ) = 0 ) Then  //  Is NT/2000
    Begin
		scm := OpenSCManager( Nil, Nil, SC_MANAGER_CONNECT );
    	svc := OpenService( scm, PChar( aServiceName ), SERVICE_ALL_ACCESS );
	    If ( svc <> 0 ) Then
    	Begin
			QueryServiceStatus( svc, sts );
            If ( aOp = -1 ) Then
            Begin
                Result := sts.dwCurrentState = SERVICE_RUNNING;
            End
            Else If ( ( ( aOp = 0 ) AND ( sts.dwCurrentState <> SERVICE_STOPPED ) )
            	OR ( ( aOp = 1 ) AND ( sts.dwCurrentState <> SERVICE_RUNNING ) ) ) Then
            Begin
				arg := Nil;
	        	If ( aOp = 0 ) Then  //  Stop
    		       	Result := ControlService( svc, SERVICE_CONTROL_STOP, sts )
            	Else
            		Result := StartService( svc, 0, arg );
            End;
		   	CloseServiceHandle( svc );
		End;
    	CloseServiceHandle( scm );
    End;
end;

Procedure GetVisibleRect( Var aRect : TRect );
Begin
    GetWindowRect( FindWindow( 'Shell_TrayWnd', '' ), aRect );  //  ȡС
    If ( aRect.Left > 0 ) Then
    Begin
    	aRect.Right := aRect.Left;
        aRect.Left  := 0;
    End
	Else If ( aRect.Top > 0 ) Then
    Begin
    	aRect.Bottom := aRect.Top;
        aRect.Top    := 0;
    End
    Else If ( aRect.Right < Screen.Width ) Then
    Begin
    	aRect.Left  := aRect.Right;
        aRect.Right := Screen.Width;
    End
    Else If ( aRect.Bottom < Screen.Height ) Then
    Begin
    	aRect.Top    := aRect.Bottom;
        aRect.Bottom := Screen.Height;
    End
    Else
    Begin
    	aRect.Left   := 0;
        aRect.Top    := 0;
        aRect.Right  := Screen.Width;
        aRect.Bottom := Screen.Height;
    End;
End;

{ TMCommandString }

constructor TMCommandString.Create;
begin
    Inherited;
    FParams := TStringList.Create;
    FParams.Delimiter := ';';
end;

destructor TMCommandString.Destroy;
begin
    FParams.Free;
    Inherited;
end;

procedure TMCommandString.LoadFromStream( aStream : TStream );
var
    strm : TStringStream;
begin
    strm := TStringStream.Create( '' );
    Try
        strm.CopyFrom( aStream, aStream.Size - aStream.Position );
        CmdStr := strm.DataString;
    Finally
        strm.Free;
    End;
end;

procedure TMCommandString.SaveToStream( aStream : TStream );
var
    strm : TStringStream;
begin
    strm := TStringStream.Create( CmdStr );
    Try
        aStream.CopyFrom( strm, strm.Size );
    Finally
        strm.Free;
    End;
end;

procedure TMCommandString.Split;
Var
    n : Integer;
begin
    FParams.Clear;
    n := Pos( ':', FCmdStr );
    If ( n > 0 ) Then
    Begin
        FCommand := Copy( FCmdStr, 1, n - 1 );
        FParams.DelimitedText := Copy( FCmdStr, n + 1, Length( FCmdStr ) - n );
    End
    Else
        FCommand := FCmdStr;
end;

procedure TMCommandString.Merge;
begin
    FCmdStr := FCommand + ':' + FParams.DelimitedText;
end;

procedure TMCommandString.SetCmdStr(const Value: String);
begin
    FCmdStr := Value;
    Split;
end;

procedure TMCommandString.SetCommand(const Value: String);
begin
    FCommand := Value;
    FParams.Clear;
    Merge;
end;

Function TMCommandString.GetCount : Integer;
begin
    Result := FParams.Count;
end;

function TMCommandString.GetParam(aName: String): String;
begin
    Result := FParams.Values[aName];
end;

procedure TMCommandString.SetParam(aName: String; const Value: String);
begin
    FParams.Values[aName] := Value;
    Merge;
end;

function TMCommandString.GetName(aIndex: Integer): String;
begin
    Result := FParams.Names[aIndex];
end;

{ TMFileVersion }

//  init
constructor TMFileVersion.Create;
Begin
	Inherited;
	FMajor   := 0;
    FMinor   := 0;
    FRelease := 0;
    FBuild   := 0;
End;

destructor TMFileVersion.Destroy;
Begin
	Inherited;
End;

//  Get version info from a file
Procedure TMFileVersion.GetVersionFromFile( aFileName : String );
Type
	PVS_FIXEDFILEINFO = ^VS_FIXEDFILEINFO;
Var
	h : Cardinal;        // a handle, ignore
    nSize : Cardinal;    // version info size
    pData : Pointer;     // version info data
    pffiData : PVS_FIXEDFILEINFO;  // fixed file info data
    nffiSize : Cardinal; // fixed file info size
Begin
	FMajor   := 0;
    FMinor   := 0;
    FRelease := 0;
    FBuild   := 0;

    If ( FileExists( aFileName ) ) Then
    	FBuild := 1;
        
	nSize := GetFileVersionInfoSize( PChar( aFileName ), h );
    If ( nSize = 0 ) Then
        Exit;

    GetMem( pData, nSize );
    Try
    	GetFileVersionInfo( PChar( aFileName ), h, nSize, pData );
        If ( VerQueryValue( pData, '\', Pointer( pffiData ), nffiSize ) ) Then
        Begin
            FMajor   := ( pffiData^.dwFileVersionMS ) SHR 16;
            FMinor   := ( pffiData^.dwFileVersionMS ) AND $FFFF;
            FRelease := ( pffiData^.dwFileVersionLS ) SHR 16;
            FBuild   := ( pffiData^.dwFileVersionLS ) AND $FFFF;
        End;
    Finally
    	FreeMem( pData );
    End;
End;

//  Compare two version info
Function TMFileVersion.Compare( aVer : TMFileVersion ) : Integer;
Var
	n1, n2 : Cardinal;
Begin
	n1 := ( FMajor SHL 16 ) OR FMinor;
    With aVer Do
	    n2 := ( FMajor SHL 16 ) OR FMinor;
    If ( n1 > n2 ) Then
    	Result := 1
    Else If ( n1 < n2 ) Then
    	Result := -1
    Else
    Begin
    	n1 := ( FRelease SHL 16 ) OR FBuild;
        With aVer Do
        	n2 := ( FRelease SHL 16 ) OR FBuild;
        If ( n1 > n2 ) Then
        	Result := 1
        Else IF ( n1 < n2 ) Then
        	Result := -1
        Else
        	Result := 0;
    End;
End;

//  Get/Set property - VerStr
Function TMFileVersion.GetVerStr : String;
Begin
	Result := Format( '%d,%.02d,%d,%.02d', [FMajor, FMinor, FRelease, FBuild] );
End;

Procedure TMFileVersion.SetVerStr( aVerStr : String );
Var
	sTemp : TStrings;
Begin
	FMajor   := 0;
    FMinor   := 0;
    FRelease := 0;
    FBuild   := 0;
    
	sTemp := TStringList.Create;
	Try
    	sTemp.CommaText := aVerStr;
        Try
		    FMajor   := StrToInt( sTemp.Strings[0] );
            FMinor   := StrToInt( sTemp.Strings[1] );
            FRelease := StrToInt( sTemp.Strings[2] );
	        FBuild   := StrToInt( sTemp.Strings[3] );
        Except
            //  Do nothing
        End;
    Finally
	    sTemp.Free;
    End;
End;

{ TMCustomApplication }

Constructor TMCustomApplication.Create;
Begin
    FHandle := Classes.AllocateHWnd( WndFunc );
End;

destructor TMCustomApplication.Destroy;
begin
    Classes.DeallocateHWnd( FHandle );
    inherited;
end;

procedure TMCustomApplication.WndFunc(var aMsg: TMessage);
begin
    aMsg.Result := DefWindowProc( FHandle, aMsg.Msg, aMsg.wParam, aMsg.lParam );
end;

end.
